'**********
'* ShowUserDept.vbs
'*
'* Display official university college/dept data (from a central LDAP server) 
'* for a given NetID.  This data is used to determine the OU location for a
'* user (if the OU manager has stepped forward for their dept for establish
'* the OU).
'*
'* Important! College/Dept data is only available for faculty/staff - NOT students!
'*
'* Command Syntax
'*
'*   Interactive mode (prompts for NetIDs, one by one):
'*
'*     showuserdept
'*
'*   Batch mode (NetIDs in input .txt file, results to output .txt file)
'*
'*     showuserdept <file>
'*
'*     Where: - "<file>" is the name of a ".txt" file (WITHOUT the ".txt" extention).
'*            - Path qualification allowed.  If none, uses current directory.
'*            - This file contains one NetID per line.
'*            - Output is placed in "<file>.Results.txt" output file in the same
'*              directory as the input file.
'*            - A "Done" message box will appear when processing is complete.
'*
'*     Example:
'*       showuserdept myusers
'*
'*       The file "myusers.txt" must exist in the current directory.  Results
'*       are placed in "myusers.Results.txt" (in the current directory).
'*          
'* Author:  Steven L. Kunz
'*          Office of Academic Technologies
'*          Iowa State University, Ames, Iowa (USA)
'*
'* Version History:
'*   1.0 -- 12/07/01 -- Base version
'*   1.1 -- 10/07/02 -- Minor tweek to "on error" code
'*   2.0 -- 05/18/04 -- Adjust for new version of ePlanet LDAP server
'*   2.1 -- 08/15/05 -- Adjust for H SCI college
'*   2.2 -- 08/29/05 -- Another adjust for H SCI college
'*   2.3 -- 01/06/09 -- Added college/major info for students
'*   2.4 -- 01/24/12 -- Updates for various colleges/major-operating-units
'*   3.0 -- 07/19/12 -- Updates for various colleges/major-operating-units, affiliate account sponsor
'*   3.1 -- 07/14/16 -- Updates for various colleges/major-operating-units
'*
'**********
option explicit

  dim username, inputname, inputfile
  dim i, cnt, notdone
  dim upn
  dim fso, tfo, LogFile, FileText
  dim unitname(100)
  dim sLDAPsrv, sLDAPsb
  Dim Not_In_LDAP, tag, LDAP_info

CONST CAPTION = "ShowUserDept v3.0"

CONST ADS_SCHEMA_ERROR = &H8000500C
CONST ADSTYPE_CASE_IGNORE_STRING = 3	'The string is of the case-insensitive type.

CONST ADS_SECURE_AUTHENTICATION = &H0001
CONST ADS_USE_ENCRYPTION = &H0002
CONST ADS_USE_SSL = &H0002
CONST ADS_READONLY_SERVER = &H0004
CONST ADS_NO_AUTHENTICATION = &H0010
CONST ADS_FAST_BIND = &H0020
CONST ADS_USE_SIGNING = &H0040
CONST ADS_USE_SEALING = &H0080
CONST ADS_USE_DELEGATION = &H0100
CONST ADS_SERVER_BIND = &H0200

  sLDAPsrv = "directory.iastate.edu"
  sLDAPsb = "ou=People,dc=iastate,dc=edu"

  '* Load our unitname array for the entire run
  LoadUnitName

  '* Get any command line args.
  '* If there are none, then drop into interactive mode and prompt the user for the
  '* username information.

  if Wscript.Arguments.Count < 1 then
    notdone = 1
    '* INTERACTIVE MODE
    do while notdone
      username = InputBox("Please supply an ISU NetID (a null value exits)", CAPTION, "")
      if username <> "" then
	LDAP_info = GetInfo(username)
	if Not_In_LDAP then
	  tag = VBOKOnly+VBCritical
	else
	  tag = VBOKOnly
	end if
        MsgBox LDAP_info, tag, CAPTION
      else
	notdone = 0
      end if
    loop
    wscript.quit
  end if

  '* BATCH MODE
  '* The command line argument is the name of the text file (WITHOUT the ".txt. 
  '* extentsion) containing a list of NetIDs (one per line).
  '* Output is placed in <arg>.Results.txt
 
  if Wscript.Arguments.Count > 0 then
    inputname = Wscript.Arguments(0)
    inputfile = inputname & ".txt"
  end if

  '*
  '* Set up some needed objects
  '*
  set fso = CreateObject("Scripting.FileSystemObject")
  if Not fso.FileExists(inputfile) then
    MsgBox inputfile & " -- File does not exist.", VBOKOnly+VBCritical, CAPTION
    set fso = Nothing
    wscript.quit
  end if

  '*
  '* Set up new output file
  '*
  set LogFile = fso.CreateTextFile(inputname & ".Results.txt",2)
  LogFile.WriteLine("Start: " & date & " " & time)

  '*
  '* Process all users listed in input file
  '*
  Set tfo = fso.OpenTextFile(inputfile, 1)
  cnt = 0
  Do While tfo.AtEndOfStream <> True
    FileText = tfo.ReadLine
    upn = Trim(FileText)

    LogFile.WriteLine GetInfo(upn)
    
    cnt = cnt + 1
  Loop

  tfo.Close

  '* Write out some summary stats
  LogFile.WriteLine("Stop: " & date & " " & time)
  LogFile.WriteLine " "
  LogFile.WriteLine "Processed " & cnt & " users"

  LogFile.Close

  '* Clean up and go home
  set tfo = Nothing
  set LogFile = Nothing
  set fso = Nothing

  MsgBox "Done", VBOKOnly, CAPTION
  wscript.quit

'**********
'* GetInfo -- Load LDAP info for given NetID
'*
'* Input:  String with NetID
'* Output: Formatted string with info
'**********
function GetInfo (upn)

  Dim sRoot
  Dim oDSP, oUser
  Dim dept, deptnum, unitabrv, afil, susp, sponsor
  Dim college, major

  Not_In_LDAP = FALSE

  '*
  '* Get LDAP information for a given user for college/dept/affiliation
  '*

  sRoot = "LDAP://" & sLDAPsrv & "/uid=" & upn & "," & sLDAPsb

  '* Set directory service provider
  Set oDSP = GetObject("LDAP:")

  '* Open DSObject for uid
  Err.Clear
  On Error Resume Next
  Set oUser = oDSP.OpenDSObject(sRoot,vbNullString,vbNullString, ADS_SERVER_BIND AND ADS_NO_AUTHENTICATION)
  If Err.Number Then
    '* Not in LDAP
    'wscript.echo Err.Number & " -- " & Err.Description
    Err.Clear
    on error goto 0
    dept = ""
    deptnum = ""
    unitabrv = ""
    afil = ""
    susp = ""
    Not_In_LDAP = TRUE
  Else
    on error goto 0
    Not_In_LDAP = FALSE

    '* Load attributes we need
    dept = GetAttrib(oUser,"isuPersonDeptAbrvn")
    deptnum = GetAttrib(oUser,"isuPersonDeptNum")
    college = GetAttrib(oUser,"isuPersonCollegeAbbrev")
    major = GetAttrib(oUser,"isuPersonStudentMajor")
    afil = GetAttrib(oUser,"eduPersonPrimaryAffiliation")
    susp = GetAttrib(oUser,"isuPersonStatus")
    sponsor = GetAttrib(oUser,"isuPersonSponsorsNetID")

    '* Done with all directory objects
    Set oUser = Nothing
    Set oDSP = Nothing

    '* Handle null values.
    '* If the uid is in LDAP and the value is null,
    '* then we return "<null>"
    if (dept = "") Then
      dept = "<null>"
    End If
    if (deptnum = "") Then
      deptnum = "<null>"
      unitabrv = "<null>"
    Else
      unitabrv = unitname(left(deptnum,2))
    End If
    if (afil = "") Then
      afil = "<null>"
    End If
    if (college = "") Then
      college = "<null>"
    End If
    if (major = "") Then
      major = "<null>"
    End If
    if (sponsor = "") Then
      sponsor = "<null>"
    End If
  End If

  '* Clean up values
  dept = Trim(dept)
  deptnum = Trim(deptnum)
  afil = Trim(afil)
  sponsor = Trim(sponsor)
  If (len(upn) < 8) Then
    i = 8 - len(upn)
    upn = upn & space(i)
  End If

  '* Build return string
  if Not_In_LDAP then
    GetInfo = upn & " -- Not in LDAP"
  else
    if afil = "student" then
      GetInfo = upn & " -- " & afil & " -- " & college & "/" & major
    elseif afil = "affiliate" then
      GetInfo = upn & " -- " & deptnum & " -- " & afil & " (" & sponsor & ") -- " & unitabrv & "/" & dept
    else
      GetInfo = upn & " -- " & deptnum & " -- " & afil & " -- " & unitabrv & "/" & dept
    end if
    if (susp <> "") then
      GetInfo = GetInfo & " [" & susp & "]"
    end if
  end if

end function

'**********
'* GetAttrib -- Load attribute value for given attribute
'*
'* Input:  Open DSObject, String with attribute name
'* Output: Formatted string with info
'**********
function GetAttrib (oUser,sAttribName)
  dim LastError
  dim sAttribVal
  dim entry, v, propval

  '* Try to get the attribute
  Err.Clear
  LastError = 0
  On Error Resume Next
  sAttribVal = oUser.Get(sAttribName)
  LastError = Err.Number
  If LastError <> 0 and LastError <> ADS_SCHEMA_ERROR Then
    echo "Error: " & LastError & " " & sAttribName
    GetAttrib = ""
    exit function
  End If
  on error goto 0

  '* Check for errors from latest ePlanet server
  If (LastError = ADS_SCHEMA_ERROR) then
    '* Funny error from latest ePlanet LDAP server
    oUser.GetInfoEx Array(sAttribName), ADSTYPE_CASE_IGNORE_STRING
    set entry = oUser.GetPropertyItem(sAttribName, ADSTYPE_CASE_IGNORE_STRING)
    for each v in entry.Values
      set propval = v
      sAttribVal = propval.GetObjectProperty(ADSTYPE_CASE_IGNORE_STRING)
    next
  Else
    '* Easy case
    sAttribVal = oUser.Get(sAttribName)
  End If

  GetAttrib = sAttribVal

end function

'**********
'* LoadUnitName -- Load array with College/Operating-Unit names
'*
'* This must be done because this string is not yet loaded into the LDAP server
'* for the users (but the numeric code is).
'*
'* Input:  Nothing
'* Output: "unitname" array loaded 
'**********
Sub LoadUnitName

  for i = 1 to 100
    unitname(i) = "unknown"
  next 

  unitname(01) = "AG"
  unitname(02) = "ENGR"
  unitname(04) = "LAS"
  unitname(05) = "V MED"
  unitname(07) = "DSN"
  unitname(08) = "BUS"
  unitname(09) = "LIB"
  unitname(10) = "H SCI"
  unitname(11) = "FPM"
  unitname(12) = "VPBUS"
  unitname(13) = "REGNT"
  unitname(14) = "VP SA"
  unitname(15) = "UNSRV"
  unitname(16) = "DVFIN"
  unitname(17) = "VPRED"
  unitname(18) = "SVPP"
  unitname(19) = "PRES"
  unitname(20) = "ERI"
  unitname(22) = "STATL"
  unitname(23) = "VMRI"
  unitname(25) = "IPRT"
  unitname(26) = "ITS"
  unitname(28) = "HUMSR"
  unitname(29) = "IDRO"
  unitname(30) = "C EXT"
  unitname(31) = "U EXT"
  unitname(33) = "EXT C"
  unitname(35) = "CIRAS"
  unitname(40) = "AEX S"
  unitname(45) = "A LAB"
  unitname(66) = "PSI"
  unitname(68) = "BUSRI"
  unitname(70) = "M COL"
  unitname(80) = "INTDP"
  unitname(92) = "ISURP"
  unitname(99) = "OTHER"

end sub
